//   Copyright (c)  2017  John Abbott,  Anna M. Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/RandomSmallPrime.H"
#include "CoCoA/error.H"
#include "CoCoA/NumTheory.H"
#include "CoCoA/random.H"

#include <limits>
using std::numeric_limits;
#include <vector>
using std::vector;

namespace CoCoA
{

  namespace
  {
    // Table of residues modulo 30 for numbers not divisible by 2, 3 or 5:
    const char shift30[8] = {1, 7, 11, 13, 17, 19, 23, 29};
  }

  // Fairly straightforward; handling 2, 3, 5 and 7 was a bit tedious :-/
  long RandomSmallPrime(const MachineInt& MAX)
  {
    if (IsNegative(MAX)) CoCoA_ERROR(ERR::BadArg, "RandomSmallPrime");
    const long UPB = AsSignedLong(MAX);
    // Arbitrarily impose limit  MAX <= 2^31-1
    if (numeric_limits<long>::radix == 2 && numeric_limits<long>::digits > 32 && UPB > 2147483647L)
      CoCoA_ERROR(ERR::ArgTooBig, "RandomSmallPrime");

    const long UPBOver30 = UPB/30;  // integer division!
    while (true)
    {
      const int candidate = 30*RandomLong(1, UPBOver30) + shift30[RandomLong(0,7)];
      if (candidate < 11)
      { if (candidate == 2 || candidate == 3 || candidate == 5 || candidate == 7)
          return candidate;
        else
          continue;
      }
      if (candidate <= UPB && candidate%7 && IsPrime(candidate)) return candidate;
    }
  }


} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/RandomSmallPrime.C,v 1.1 2017/11/14 14:59:35 abbott Exp $
// $Log: RandomSmallPrime.C,v $
// Revision 1.1  2017/11/14 14:59:35  abbott
// Summary: New fns: RandomSmallPrime, RandomSparseNonSing01Matrix
//
//
